@@ -5,9 +5,9 @@
 ###
 ### Some internal helper functions for "twinstim".
 ###
-### Copyright (C) 2009-2015 Sebastian Meyer
-### $Revision: 1285 $
-### $Date: 2015-03-24 15:26:51 +0100 (Tue, 24 Mar 2015) $
+### Copyright (C) 2009-2016,2018 Sebastian Meyer
+### $Revision: 2100 $
+### $Date: 2018-04-12 16:51:42 +0200 (Thu, 12. Apr 2018) $
 ################################################################################
 
 
@@ -17,7 +17,7 @@
 ## all arguments but i and qmatrix are nEvents-vectors
 ## -> determine potential sources for eventTimes[i], eventsTypes[i] with
 ## distances distvec_j = ||s_i - s_j||
-determineSources <- function (i, eventTimes, removalTimes, distvec, eps.s,
+determineSources1 <- function (i, eventTimes, removalTimes, distvec, eps.s,
     eventTypes = NULL, qmatrix)
 {
     tp <- eventTimes[i]
@@ -37,18 +37,52 @@ determineSources <- function (i, eventTimes, removalTimes, distvec, eps.s,
     unname(sources)
 }
 
-## determine the .sources for an epidataCS object, i.e.
-## lapply the previous function to all of object$events
-determineSources.epidataCS <- function (object)
+## interface to new C++ implementation (including a loop over all events)
+determineSources <- function (eventTimes, eps.t,
+                              eventCoords, eps.s,
+                              eventTypes, qmatrix)
 {
-    eventTimes <- object$events@data$time
-    removalTimes <- eventTimes + object$events@data$eps.t
-    eventDists <- as.matrix(dist(object$events@coords, method = "euclidean"))
-    lapply(seq_along(eventTimes), function (i) {
-        determineSources(i, eventTimes, removalTimes, eventDists[i,],
-                         object$events@data$eps.s, object$events@data$type,
-                         object$qmatrix) 
-    })
+    ## check inputs
+    stopifnot(is.vector(eventTimes, mode = "numeric"),
+              is.vector(eps.t, mode = "numeric"),
+              is.matrix(eventCoords), is.numeric(eventCoords),
+              ncol(eventCoords) == 2L, is.vector(eps.s, mode = "numeric"),
+              is.matrix(qmatrix), is.logical(qmatrix),
+              nrow(qmatrix) == ncol(qmatrix))
+    N <- length(eventTimes)
+    stopifnot(nrow(eventCoords) == N, length(eventTypes) == N)
+    if (is.factor(eventTypes)) {
+        stopifnot(nlevels(eventTypes) <= nrow(qmatrix))
+    } else {
+        stopifnot(is.vector(eventTypes, mode = "integer"),
+                  max(eventTypes) <= nrow(qmatrix))
+    }
+
+    ## call C++ function (wrapper)
+    determineSourcesC(eventTimes = eventTimes, eps_t = rep_len(eps.t, N),
+                      eventCoords = eventCoords, eps_s = rep_len(eps.s, N),
+                      eventTypes = as.integer(eventTypes), qmatrix = qmatrix)
+}
+
+## determine the .sources for an epidataCS object
+determineSources.epidataCS <- function (object, method = c("C", "R"))
+{
+    method <- match.arg(method)
+    if (method == "R") {
+        ## lapply determineSources1() to all of object$events
+        eventTimes <- object$events@data$time
+        removalTimes <- eventTimes + object$events@data$eps.t
+        eventDists <- as.matrix(dist(object$events@coords, method = "euclidean"))
+        lapply(seq_along(eventTimes), function (i) {
+            determineSources1(i, eventTimes, removalTimes, eventDists[i,],
+                              object$events@data$eps.s, object$events@data$type,
+                              object$qmatrix)
+        })
+    } else {
+        determineSources(eventTimes = object$events$time, eps.t = object$events$eps.t,
+                         eventCoords = coordinates(object$events), eps.s = object$events$eps.s,
+                         eventTypes = object$events$type, qmatrix = object$qmatrix)
+    }
 }
 
 
@@ -85,18 +119,19 @@ checkQ <- function (qmatrix, typeNames)
 gridcellOfEvent <- function (t, tilename, stgrid)
 {
     ## idx <- with(stgrid, which(tile == tilename & start < t & stop >= t))
-    
+    ## lidx <- length(idx)
+    ## if (lidx == 0L) NA_integer_ else if (lidx == 1L) idx else {
+    ##     stop("'stgrid' has overlapping spatio-temporal grid cells")
+    ## }
+
     ## ~5x faster alternative assuming a full BLOCK x tile grid, which is
     ## sorted by BLOCK and tile (tile varying first), specifically there must be
     ## all levels(stgrid$tile) in every BLOCK in that order;
     ## this structure is guaranteed by check_stgrid()
-    blockstart <- match(TRUE, stgrid$stop >= t)
+    if (t <= stgrid$start[1L]) return(NA_integer_)  # prehistory event
+    blockstart <- match(TRUE, stgrid$stop >= t)     # NA if t is beyond
     idx <- blockstart + match(tilename, levels(stgrid$tile)) - 1L
-    
-    lidx <- length(idx)
-    if (lidx == 0L) NA_integer_ else if (lidx == 1L) idx else {
-        stop("'stgrid' has overlapping spatio-temporal grid cells")
-    }
+    return(idx)
 }
 
 
@@ -124,10 +159,10 @@ crudebeta0 <- function (nEvents, offset.mean, W.area, period, nTypes)
 ){
     ## the following variables are unused here, because the environment of
     ## FUN will be set to the parent.frame(), where the variables exist
-    ## they are only included to avoid the notes in R CMD check 
+    ## they are only included to avoid the notes in R CMD check
     iRareas <- influenceRegion <- eventTypes <- eps.s <- bdist <- effRanges <- NULL
-    
-    ## define the siaf integration function depending on the siaf specification 
+
+    ## define the siaf integration function depending on the siaf specification
     FUN <- if (attr(siaf, "constant"))
     {
         if (exists("iRareas", where=parent.frame(), mode="numeric")) {
@@ -182,7 +217,7 @@ crudebeta0 <- function (nEvents, offset.mean, W.area, period, nTypes)
             effRangeTypes <- rep_len(siaf$effRange(siafpars), nTypes),
             effRanges <- effRangeTypes[eventTypes]   # N-vector
             ),
-            parallel = parallel)        
+            parallel = parallel)
         if (exists("effRangeTypes", where=parent.frame(), mode="numeric")) {
             ## in simEpidataCS effRangeTypes is pre-calculated outside siafInt to
             ## save computation time ('siafpars' is constant during simulation)
@@ -190,7 +225,7 @@ crudebeta0 <- function (nEvents, offset.mean, W.area, period, nTypes)
         }
         .ret
     }
-    
+
     ## set the environment of the siafInt function to the callers environment
     ## (i.e. inside twinstim() or simEpidataCS())
     ## where the variables used in the function are defined
@@ -208,9 +243,9 @@ crudebeta0 <- function (nEvents, offset.mean, W.area, period, nTypes)
 {
     ## the following variables are unused here, because the environment of
     ## FUN will be set to the parent.frame(), where the variables exist
-    ## they are only included to avoid the notes in R CMD check 
+    ## they are only included to avoid the notes in R CMD check
     gIntLower <- gIntUpper <- eventTypes <- tiaf <- NULL
-    
+
     ## from, to and type may be vectors of compatible lengths
     FUN <- function(tiafpars, from = gIntLower, to = gIntUpper,
                     type = eventTypes, G = tiaf$G)
@@ -219,7 +254,7 @@ crudebeta0 <- function (nEvents, offset.mean, W.area, period, nTypes)
         tiafIntLower <- G(from, tiafpars, type)
         tiafIntUpper - tiafIntLower
     }
-    
+
     ## set the environment of the tiafInt function to the callers environment
     ## (i.e. inside twinstim() or simEpidataCS())
     ## where the default argument values are defined
